@@ -1,4 +1,4 @@
-# Copyright 2020 The TensorFlow Recommenders Authors.
+# Copyright 2021 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,9 @@
 # Lint-as: python3
 """Layers."""
 
-from tensorflow_recommenders.layers import dcn
+from tensorflow_recommenders.layers import blocks
+from tensorflow_recommenders.layers import embedding
 from tensorflow_recommenders.layers import factorized_top_k
+from tensorflow_recommenders.layers import feature_interaction
 from tensorflow_recommenders.layers import loss
+from tensorflow_recommenders.layers.feature_interaction import dcn
